@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2008  The PARI group.
+/* Copyright (C) 2008  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -91,38 +89,6 @@ lll_finish(GEN h, long k, long flag)
   http://www.shoup.net/ntl/
 */
 
-/* x - s*y */
-static GEN
-submulu(GEN x, GEN y, ulong s)
-{
-  pari_sp av;
-  if (!signe(y)) return x;
-  av = avma;
-  (void)new_chunk(3+lgefint(x)+lgefint(y)); /* HACK */
-  y = mului(s,y);
-  avma = av; return subii(x, y);
-}
-static GEN
-addmulu(GEN x, GEN y, ulong s)
-{
-  pari_sp av;
-  if (!signe(y)) return x;
-  av = avma;
-  (void)new_chunk(3+lgefint(x)+lgefint(y)); /* HACK */
-  y = mului(s,y);
-  avma = av; return addii(x, y);
-}
-static GEN
-submul(GEN x, GEN y, GEN s)
-{
-  pari_sp av;
-  if (!signe(y)) return x;
-  av = avma;
-  (void)new_chunk(lgefint(s)+lgefint(x)+lgefint(y)); /* HACK */
-  y = mulii(s,y);
-  avma = av; return subii(x, y);
-}
-
 /***********************************************/
 /* Babai's Nearest Plane algorithm (iterative) */
 /***********************************************/
@@ -258,20 +224,20 @@ Babai(pari_sp av, long kappa, GEN *pG, GEN *pB, GEN *pU, GEN mu, GEN r, GEN s,
           }
           avma = btop;
           for (i=1; i<=n; i++)
-            gmael(B,kappa,i) = submulu(gmael(B,kappa,i), gmael(B,j,i), xx);
+            gmael(B,kappa,i) = submuliu_inplace(gmael(B,kappa,i), gmael(B,j,i), xx);
           for (i=1; i<=d; i++)
-            gmael(U,kappa,i) = submulu(gmael(U,kappa,i), gmael(U,j,i), xx);
+            gmael(U,kappa,i) = submuliu_inplace(gmael(U,kappa,i), gmael(U,j,i), xx);
           btop = avma;
           ztmp = shifti(muliu(gmael(G,kappa,j), xx), 1);
           ztmp = subii(mulii(gmael(G,j,j), sqru(xx)), ztmp);
           ztmp = addii(gmael(G,kappa,kappa), ztmp);
           gmael(G,kappa,kappa) = gerepileuptoint(btop, ztmp);
           for (i=1; i<=j; i++)
-            gmael(G,kappa,i) = submulu(gmael(G,kappa,i), gmael(G,j,i), xx);
+            gmael(G,kappa,i) = submuliu_inplace(gmael(G,kappa,i), gmael(G,j,i), xx);
           for (i=j+1; i<kappa; i++)
-            gmael(G,kappa,i) = submulu(gmael(G,kappa,i), gmael(G,i,j), xx);
+            gmael(G,kappa,i) = submuliu_inplace(gmael(G,kappa,i), gmael(G,i,j), xx);
           for (i=kappa+1; i<=maxG; i++)
-            gmael(G,i,kappa) = submulu(gmael(G,i,kappa), gmael(G,i,j), xx);
+            gmael(G,i,kappa) = submuliu_inplace(gmael(G,i,kappa), gmael(G,i,j), xx);
         }
         else /* = -xx */
         {
@@ -282,27 +248,27 @@ Babai(pari_sp av, long kappa, GEN *pG, GEN *pB, GEN *pU, GEN mu, GEN r, GEN s,
           }
           avma = btop;
           for (i=1; i<=n; i++)
-            gmael(B,kappa,i) = addmulu(gmael(B,kappa,i), gmael(B,j,i), xx);
+            gmael(B,kappa,i) = addmuliu_inplace(gmael(B,kappa,i), gmael(B,j,i), xx);
           for (i=1; i<=d; i++)
-            gmael(U,kappa,i) = addmulu(gmael(U,kappa,i), gmael(U,j,i), xx);
+            gmael(U,kappa,i) = addmuliu_inplace(gmael(U,kappa,i), gmael(U,j,i), xx);
           btop = avma;
           ztmp = shifti(muliu(gmael(G,kappa,j), xx), 1);
           ztmp = addii(mulii(gmael(G,j,j), sqru(xx)), ztmp);
           ztmp = addii(gmael(G,kappa,kappa), ztmp);
           gmael(G,kappa,kappa) = gerepileuptoint(btop, ztmp);
           for (i=1; i<=j; i++)
-            gmael(G,kappa,i) = addmulu(gmael(G,kappa,i), gmael(G,j,i), xx);
+            gmael(G,kappa,i) = addmuliu_inplace(gmael(G,kappa,i), gmael(G,j,i), xx);
           for (i=j+1; i<kappa; i++)
-            gmael(G,kappa,i) = addmulu(gmael(G,kappa,i), gmael(G,i,j), xx);
+            gmael(G,kappa,i) = addmuliu_inplace(gmael(G,kappa,i), gmael(G,i,j), xx);
           for (i=kappa+1; i<=maxG; i++)
-            gmael(G,i,kappa) = addmulu(gmael(G,i,kappa), gmael(G,i,j), xx);
+            gmael(G,i,kappa) = addmuliu_inplace(gmael(G,i,kappa), gmael(G,i,j), xx);
         }
       }
       else
       {
         GEN tmp2  = itor(ztmp,prec);
-        long expo = expo(tmp2)-bit_accuracy(prec);
-        GEN X = shifti(trunc2nr(tmp2, -expo), expo);
+        long e = expo(tmp2)-prec2nbits(prec);
+        GEN X = shifti(trunc2nr(tmp2, -e), e);
         pari_sp btop = avma;
 
         for (k=zeros+1; k<j; k++)
@@ -312,20 +278,20 @@ Babai(pari_sp av, long kappa, GEN *pG, GEN *pB, GEN *pU, GEN mu, GEN r, GEN s,
         }
         avma = btop;
         for (i=1; i<=n; i++)
-          gmael(B,kappa,i) = submul(gmael(B,kappa,i), gmael(B,j,i), X);
+          gmael(B,kappa,i) = submulii(gmael(B,kappa,i), gmael(B,j,i), X);
         for (i=1; i<=d; i++)
-          gmael(U,kappa,i) = submul(gmael(U,kappa,i), gmael(U,j,i), X);
+          gmael(U,kappa,i) = submulii(gmael(U,kappa,i), gmael(U,j,i), X);
         btop = avma;
         ztmp = shifti(mulii(gmael(G,kappa,j), X), 1);
         ztmp = subii(mulii(gmael(G,j,j), sqri(X)), ztmp);
         ztmp = addii(gmael(G,kappa,kappa), ztmp);
         gmael(G,kappa,kappa) = gerepileuptoint(btop, ztmp);
         for (i=1; i<=j; i++)
-          gmael(G,kappa,i) = submul(gmael(G,kappa,i), gmael(G,j,i), X);
+          gmael(G,kappa,i) = submulii(gmael(G,kappa,i), gmael(G,j,i), X);
         for (   ; i<kappa; i++)
-          gmael(G,kappa,i) = submul(gmael(G,kappa,i), gmael(G,i,j), X);
+          gmael(G,kappa,i) = submulii(gmael(G,kappa,i), gmael(G,i,j), X);
         for (i=kappa+1; i<=maxG; i++)
-          gmael(G,i,kappa) = submul(gmael(G,i,kappa), gmael(G,i,j), X);
+          gmael(G,i,kappa) = submulii(gmael(G,i,kappa), gmael(G,i,j), X);
       }
     }
     if (!go_on) break; /* Anything happened? */
@@ -384,7 +350,7 @@ fplll(GEN *ptrB, GEN *ptrU, GEN *ptrr, double DELTA, double ETA, long flag, long
   else
   {
     G = zeromatcopy(d,d);
-    n = lg(gel(B,1))-1;
+    n = nbrows(B);
   }
   U = *ptrU; /* NULL if inplace */
 
@@ -455,7 +421,7 @@ fplll(GEN *ptrB, GEN *ptrU, GEN *ptrr, double DELTA, double ETA, long flag, long
     }
     else
     { /* Step5: Find the right insertion index kappa, kappa2 = initial kappa */
-      if (DEBUGLEVEL>=4 && kappa==kappamax && signe(gel(s,kappa-1)))
+      if (DEBUGLEVEL>=10 && kappa==kappamax && signe(gel(s,kappa-1)))
       {
         long e = expo(divrr(mulrr(gmael(r,kappa-1,kappa-1), delta),
                             gel(s,kappa-1)));
@@ -544,7 +510,7 @@ ZM_lll_norms(GEN x, double DELTA, long flag, GEN *B)
   if (n <= 1) return lll_trivial(x, flag);
   x = RgM_shallowcopy(x);
   U = (flag & LLL_INPLACE)? NULL: matid(n);
-  for (p = 3; ; p++)
+  for (p = LOWDEFAULTPREC; ; incrprec(p))
   {
     GEN m = fplll(&x, &U, B, DELTA, ETA, flag, p);
     if (m) return m;
@@ -646,7 +612,7 @@ incrementalGSgen(GEN x, GEN L, GEN B, long k, GEN fl)
     if (j==k || fl[j])
     {
       u = gcoeff(x,k,j); tu = typ(u);
-      if (! is_extscalar_t(tu)) pari_err(typeer,"incrementalGSgen");
+      if (! is_extscalar_t(tu)) pari_err_TYPE("incrementalGSgen",u);
       for (i=1; i<j; i++)
         if (fl[i])
         {
@@ -671,7 +637,7 @@ lllgramallgen(GEN x, long flag)
   int flc;
 
   n = lx-1; if (n<=1) return lll_trivial(x,flag);
-  if (lg(x[1]) != lx) pari_err(mattype1,"lllgramallgen");
+  if (lgcols(x) != lx) pari_err_DIM("lllgramallgen");
 
   fl = cgetg(lx, t_VECSMALL);
 
@@ -743,7 +709,7 @@ rescale_to_int(GEN x)
   int exact = 1;
 
   lx = lg(x); if (lx == 1) return x;
-  hx = lg(x[1]);
+  hx = lgcols(x);
   emin = HIGHEXPOBIT;
   for (j = 1; j < lx; j++)
     for (i = 1; i < hx; i++)
@@ -754,7 +720,7 @@ rescale_to_int(GEN x)
         case t_REAL:
           exact = 0;
           if (!signe(c)) continue;
-          e = expo(c) - bit_accuracy(lg(c));
+          e = expo(c) - bit_prec(c);
           break;
         case t_INT:
           if (!signe(c)) continue;
@@ -765,7 +731,7 @@ rescale_to_int(GEN x)
           if (exact) D = lcmii(D, gel(c,2));
           break;
         default:
-          pari_err(typeer,"rescale_to_int");
+          pari_err_TYPE("rescale_to_int",c);
           return NULL; /* not reached */
       }
       if (e < emin) emin = e;
@@ -774,15 +740,13 @@ rescale_to_int(GEN x)
   return grndtoi(gmul2n(x, -emin), &e);
 }
 
-/* If gram = 1, x = Gram(b_i), x = (b_i) otherwise
- * Quality ratio = D in ]0.25, 1[. Suggested values: 0.75 or 0.99 */
 GEN
 lllfp(GEN x, double D, long flag)
 {
   long n = lg(x)-1;
   pari_sp av = avma;
   GEN h;
-  if (n <= 1) return matid(n);
+  if (n <= 1) return lll_trivial(x,flag);
   h = ZM_lll(rescale_to_int(x), D, flag);
   return gerepilecopy(av, h);
 }
@@ -795,7 +759,7 @@ lll(GEN x) { return lllfp(x,LLLDFT,LLL_IM); }
 GEN
 qflll0(GEN x, long flag)
 {
-  if (typ(x) != t_MAT) pari_err(typeer,"qflll");
+  if (typ(x) != t_MAT) pari_err_TYPE("qflll",x);
   switch(flag)
   {
     case 0: return lll(x);
@@ -804,7 +768,7 @@ qflll0(GEN x, long flag)
     case 4: RgM_check_ZM(x,"qflll"); return lllkerim(x);
     case 5: return lllkerimgen(x);
     case 8: return lllgen(x);
-    default: pari_err(flagerr,"qflll");
+    default: pari_err_FLAG("qflll");
   }
   return NULL; /* not reached */
 }
@@ -812,7 +776,7 @@ qflll0(GEN x, long flag)
 GEN
 qflllgram0(GEN x, long flag)
 {
-  if (typ(x) != t_MAT) pari_err(typeer,"qflllgram");
+  if (typ(x) != t_MAT) pari_err_TYPE("qflllgram",x);
   switch(flag)
   {
     case 0: return lllgram(x);
@@ -820,7 +784,7 @@ qflllgram0(GEN x, long flag)
     case 4: RgM_check_ZM(x,"qflllgram"); return lllgramkerim(x);
     case 5: return lllgramkerimgen(x);
     case 8: return lllgramgen(x);
-    default: pari_err(flagerr,"qflllgram");
+    default: pari_err_FLAG("qflllgram");
   }
   return NULL; /* not reached */
 }
@@ -850,13 +814,13 @@ kerint(GEN x)
 GEN
 matkerint0(GEN x, long flag)
 {
-  if (typ(x) != t_MAT) pari_err(typeer,"matkerint");
+  if (typ(x) != t_MAT) pari_err_TYPE("matkerint",x);
   RgM_check_ZM(x, "kerint");
   switch(flag)
   {
     case 0: return kerint(x);
     case 1: return kerint1(x);
-    default: pari_err(flagerr,"matkerint");
+    default: pari_err_FLAG("matkerint");
   }
   return NULL; /* not reached */
 }
