@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000-2003  The PARI group.
+/* Copyright (C) 2000-2003  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -48,6 +46,16 @@ vec_is1to1(GEN v)
   return 1;
 }
 
+GEN
+vec_insert(GEN v, long n, GEN x)
+{
+  long i, l=lg(v);
+  GEN V = cgetg(l+1,t_VEC);
+  for(i=1; i<n; i++) gel(V,i) = gel(v,i);
+  gel(V,n) = x;
+  for(i=n+1; i<=l; i++) gel(V,i) = gel(v,i-1);
+  return V;
+}
 /*************************************************************************/
 /**                                                                     **/
 /**                   Routines for handling VECSMALL                    **/
@@ -199,6 +207,12 @@ vecvecsmall_sort(GEN x)
   return gen_sort(x, (void*)&vecsmall_lexcmp, cmp_nodata);
 }
 
+GEN
+vecvecsmall_sort_uniq(GEN x)
+{
+  return gen_sort_uniq(x, (void*)&vecsmall_lexcmp, cmp_nodata);
+}
+
 GEN
 vecvecsmall_indexsort(GEN x)
 {
@@ -285,7 +299,7 @@ perm_order(GEN v)
   pari_sp ltop = avma;
   GEN c = vecperm_orbits_i(mkvec(v), lg(v)-1);
   long i, d;
-  for(i=1, d=1; i<lg(c); i++) d = clcm(d, lg(c[i])-1);
+  for(i=1, d=1; i<lg(c); i++) d = clcm(d, lg(gel(c,i))-1);
   avma = ltop; return d;
 }
 
@@ -296,7 +310,7 @@ cyc_pow(GEN cyc, long exp)
   GEN c;
   for (r = j = 1; j < lg(cyc); j++)
   {
-    long n = lg(cyc[j]) - 1;
+    long n = lg(gel(cyc,j)) - 1;
     r += cgcd(n, exp);
   }
   c = cgetg(r, t_VEC);
@@ -325,7 +339,7 @@ cyc_pow_perm(GEN cyc, long exp)
 {
   long e, j, k, l, n;
   GEN p;
-  for (n = 0, j = 1; j < lg(cyc); j++) n += lg(cyc[j])-1;
+  for (n = 0, j = 1; j < lg(cyc); j++) n += lg(gel(cyc,j))-1;
   p = cgetg(n + 1, t_VECSMALL);
   for (j = 1; j < lg(cyc); j++)
   {
@@ -359,7 +373,7 @@ perm_to_GAP(GEN p)
   char *s;
   long sz;
   long lp=lg(p)-1;
-  if (typ(p) != t_VECSMALL)  pari_err(typeer, "perm_to_GAP");
+  if (typ(p) != t_VECSMALL)  pari_err_TYPE("perm_to_GAP",p);
   x = perm_cycles(p);
   sz = (long) ((bfffo(lp)+1) * LOG10_2 + 1);
   /*Dry run*/
@@ -462,7 +476,7 @@ long
 group_domain(GEN G)
 {
   GEN gen = grp_get_gen(G);
-  if (lg(gen) < 2) pari_err(talker,"empty group in group_domain");
+  if (lg(gen) < 2) pari_err_DOMAIN("group_domain", "#G", "=", gen_1,G);
   return lg(gel(gen,1)) - 1;
 }
 
@@ -564,29 +578,17 @@ tableset_find_index(GEN tbl, GEN set)
 }
 
 GEN
-trivialgroup(void)
-{
-  GEN G = cgetg(3, t_VEC);
-  gel(G,1) = cgetg(1,t_VEC);
-  gel(G,2) = cgetg(1,t_VECSMALL); return G;
-}
+trivialgroup(void) { retmkvec2(cgetg(1,t_VEC), cgetg(1,t_VECSMALL)); }
 /*Cyclic group generated by g of order s*/
 GEN
 cyclicgroup(GEN g, long s)
-{
-  GEN G = cgetg(3, t_VEC);
-  gel(G,1) = mkvec( vecsmall_copy(g) );
-  gel(G,2) = mkvecsmall(s); return G;
-}
+{ retmkvec2(mkvec( vecsmall_copy(g) ),
+            mkvecsmall(s)); }
 /*Return the group generated by g1,g2 of relative orders s1,s2*/
 GEN
 dicyclicgroup(GEN g1, GEN g2, long s1, long s2)
-{
-  GEN G = cgetg(3, t_VEC);
-  gel(G,1) = mkvec2(vecsmall_copy(g1), vecsmall_copy(g2));
-  gel(G,2) = mkvecsmall2(s1, s2);
-  return G;
-}
+{ retmkvec2( mkvec2(vecsmall_copy(g1), vecsmall_copy(g2)),
+             mkvecsmall2(s1, s2) ); }
 
 /* return the quotient map G --> G/H */
 /*The ouput is [gen,hash]*/
@@ -604,8 +606,8 @@ group_quotient(GEN G, GEN H)
   GEN used = zero_F2v(le+1);
   long l = le/o;
   p2 = cgetg(l+1, t_VEC);
-  p3 = const_vecsmall(n, 0);
-  el = const_vecsmall(n, 0);
+  p3 = zero_zv(n);
+  el = zero_zv(n);
   for (i = 1; i<=le; i++)
     el[mael(elt,i,1)]=i;
   for (i = 1; i <= l; ++i)
@@ -617,7 +619,7 @@ group_quotient(GEN G, GEN H)
     for(j=1;j<lg(V);j++)
     {
       long b = el[mael(V,j,1)];
-      if (b==0) pari_err(talker, "not a WSS group");
+      if (b==0) pari_err_IMPL("group_quotient for a non-WSS group");
       F2v_set(used,b);
     }
     for (j = 1; j <= o; j++)
@@ -637,7 +639,7 @@ quotient_perm(GEN C, GEN p)
   for (j = 1; j < l; ++j)
   {
     p3[j] = coset[p[mael(gen,j,1)]];
-    if (p3[j]==0) pari_err(talker,"not a WSS group");
+    if (p3[j]==0) pari_err_IMPL("quotient_perm for a non-WSS group");
   }
   return p3;
 }
@@ -892,7 +894,7 @@ group_abelianHNF(GEN G, GEN S)
     for(j=1; j<lg(S); j++)
       if (zv_equal(P, gel(S,j))) break;
     avma = av;
-    if (j==lg(S)) pari_err(talker,"wrong argument in galoisisabelian");
+    if (j==lg(S)) pari_err_BUG("galoisisabelian [inconsistent group]");
     j--;
     for(k=1; k<i; k++)
     {
@@ -919,11 +921,10 @@ group_abelianSNF(GEN G, GEN L)
 GEN
 abelian_group(GEN v)
 {
-  long card, i, d = 1, l = lg(v);
+  long card = zv_prod(v), i, d = 1, l = lg(v);
   GEN G = cgetg(3,t_VEC), gen = cgetg(l,t_VEC);
   gel(G,1) = gen;
   gel(G,2) = vecsmall_copy(v);
-  card = group_order(G);
   for(i=1; i<l; i++)
   {
     GEN p = cgetg(card+1, t_VECSMALL);
@@ -942,14 +943,15 @@ abelian_group(GEN v)
   return G;
 }
 
-/*return 1 if G is abelian, else 0*/
+/*return 1 if H is a normal subgroup of G*/
 long
 group_subgroup_isnormal(GEN G, GEN H)
 {
   GEN g = grp_get_gen(G);
   long i, n = lg(g);
   if (lg(grp_get_gen(H)) > 1 && group_domain(G) != group_domain(H))
-    pari_err(talker,"not a subgroup in group_subgroup_isnormal");
+    pari_err_DOMAIN("group_subgroup_isnormal","domain(H)","!=",
+                    strtoGENstr("domain(G)"), H);
   for(i=1; i<n; i++)
     if (!group_perm_normalize(H, gel(g,i))) return 0;
   return 1;
@@ -1007,10 +1009,10 @@ group_export_GAP(GEN G)
   GEN s, comma, g = grp_get_gen(G);
   long i, k, l = lg(g);
   if (l == 1) return strtoGENstr("Group(())");
-  s = cgetg(2*l, t_VEC); k = 2;
+  s = cgetg(2*l, t_VEC);
   comma = strtoGENstr(", ");
   gel(s,1) = strtoGENstr("Group(");
-  for (i = 1; i < l; ++i)
+  for (i=1, k=2; i < l; ++i)
   {
     if (i > 1) gel(s,k++) = comma;
     gel(s,k++) = perm_to_GAP(gel(g,i));
@@ -1026,16 +1028,13 @@ group_export_MAGMA(GEN G)
   GEN s, comma, g = grp_get_gen(G);
   long i, k, l = lg(g);
   if (l == 1) return strtoGENstr("PermutationGroup<1|>");
-  s = cgetg(2*l+2, t_VEC); k = 1;
-  gel(s,k++) = strtoGENstr("PermutationGroup<");
-  gel(s,k++) = strtoGENstr( itostr( stoi(group_domain(G)) ) );
-  gel(s,k++) = strtoGENstr("|"); comma = strtoGENstr(", ");
-  for (i = 1; i < l; ++i)
+  s = cgetg(2*l, t_VEC);
+  comma = strtoGENstr(", ");
+  gel(s,1) = gsprintf("PermutationGroup<%ld|",group_domain(G));
+  for (i=1, k=2; i < l; ++i)
   {
-    char *t = GENtostr( vecsmall_to_vec(gel(g,i)) );
     if (i > 1) gel(s,k++) = comma;
-    gel(s,k++) = strtoGENstr(t);
-    pari_free(t);
+    gel(s,k++) = GENtoGENstr( vecsmall_to_vec(gel(g,i)) );
   }
   gel(s,k++) = strtoGENstr(">");
   return gerepilecopy(av, shallowconcat1(s));
@@ -1049,6 +1048,6 @@ group_export(GEN G, long format)
   case 0: return group_export_GAP(G);
   case 1: return group_export_MAGMA(G);
   }
-  pari_err(flagerr,"galoisexport");
+  pari_err_FLAG("galoisexport");
   return NULL; /*-Wall*/
 }
