@@ -277,7 +277,7 @@ gacos(GEN x, long prec)
       if (lg(y) > 2)
       {
         p1 = gsubsg(1,gsqr(y));
-        if (gequal0(p1)) return zeroser(varn(y), valp(p1)>>1);
+        if (gequal0(p1)) { avma = av; return zeroser(varn(y), valp(p1)>>1); }
         p1 = integser(gdiv(gneg(derivser(y)), gsqrt(p1,prec)));
         /*y(t) = 1+O(t)*/
         if (gequal1(gel(y,2)) && !valp(y)) return gerepileupto(av, p1);
@@ -1708,7 +1708,8 @@ serpsi(GEN y, long prec)
   else
   {
     GEN t;
-    z0 = gel(y,2); t = ground(z0);
+    z0 = simplify_shallow(gel(y,2));
+    t = ground(z0);
     if (gequal(t,z0)) z0 = t;
   }
   reflect = (gcmp(real_i(z0),ghalf) < 0); /* use reflection formula */
